#include "stdlib.h"
#include "AMDiS.h"
#include "MultiGridWrapper.h"

//using namespace ost;
using namespace std;
using namespace AMDiS;

// ===== function definitions // 
/** \brief
 * Dirichlet boundary function
 */
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(G);

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const
  {
    static double result;
    result = exp(-10.0*(x*x));
    return result;  
  };
};

/** \brief
 * RHS function
 */
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(F);

  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {};

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const {
    static double result = 0.0;
    int dim = x.getSize();
    double r2 = (x*x);
    double ux = exp(-10.0*r2);
    result = -(400.0*r2 - 20.0*dim)*ux;
    //    result = 1000 * sin(2*M_PI*x[0])*cos(10*2*M_PI*x[1]);
    return result;
  };
};

// ===== main program // 
int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: multigrid initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create and init the scalar problem ===== 
  ProblemScal multigrid("multigrid");
  multigrid.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = NEW AdaptInfo("multigrid->adapt");

  // === create adapt ===
  AdaptStationary *adapt = NEW AdaptStationary("multigrid->adapt",
					       &multigrid,
					       adaptInfo);
  
  //multigrid.setAdapt(adapt);

  // ===== add boundary conditions =====
  multigrid.addDirichletBC(1, NEW G);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR, multigrid.getFESpace());
  matrixOperator.addSecondOrderTerm(NEW Laplace_SOT);
  multigrid.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  int degree = multigrid.getFESpace()->getBasisFcts()->getDegree();
  Operator rhsOperator(Operator::VECTOR_OPERATOR, multigrid.getFESpace());
  rhsOperator.addZeroOrderTerm(NEW CoordsAtQP_ZOT(NEW F(degree)));
  multigrid.addVectorOperator(&rhsOperator);

  adapt->adapt();

  multigrid.writeFiles(adaptInfo, true);
}


